package de.fraunhofer.iem.dataprovider.kpi.strategy

import de.fraunhofer.iem.dataprovider.kpi.dto.KPIHierarchyEdgeDto

class AggregationKPICalculationStrategy : KPICalculationStrategy {

    /**
     * This function calculates the aggregate sum of all given children.
     * If a child is empty it is removed from the calculation and its
     * corresponding edge weight is distributed evenly between the
     * remaining children.
     */
    override fun calculateKPI(children: List<KPIHierarchyEdgeDto>): Int {
        val emptyChildren: List<KPIHierarchyEdgeDto> = children.filter { it.to.isEmpty() }
        val notEmptyChildren = children.toMutableList()
        notEmptyChildren.removeAll(emptyChildren)

        val additionalWeightPerElement: Double = if (notEmptyChildren.isNotEmpty()) {
            val unusedWeight = emptyChildren.sumOf { it.weight }
            unusedWeight / notEmptyChildren.size
        } else {
            0.0
        }

        return notEmptyChildren.sumOf { child ->
            (child.to.getValue().toFloat() * (child.weight + additionalWeightPerElement)).toInt()
        }
    }
}
